@@ -27,22 +27,29 @@
 #include <tests/testcore.h>
 #include <tests/autotestshell.h>
 
-#include <QtTest/QTest>
-#include <KUrl>
-#include <KStandardDirs>
+#include <ktexteditor_version.h>
 #include <KTextEditor/Editor>
 #include <KService>
 
 #include "codecompletion/context.h"
 #include "codecompletion/helpers.h"
+#include "codecompletiondebug.h"
+
+#include <QDebug>
+#include <QStandardPaths>
+#include <QtTest/QTest>
 
 using namespace KDevelop;
 
 QTEST_MAIN(Python::PyCompletionTest)
 
+Q_DECLARE_METATYPE(QList<Python::RangeInString>)
+#if KTEXTEDITOR_VERSION < QT_VERSION_CHECK(5, 10, 0)
+Q_DECLARE_METATYPE(KTextEditor::Range)
+#endif
+
 static int testId = 0;
 static QString basepath = "/tmp/__kdevpythoncompletiontest.dir/";
-static QFSFileEngine fileEngine;
 
 namespace Python {
 
@@ -71,11 +78,10 @@ void makefile(QString filename, QString contents) {
     QFile fileptr;
     fileptr.setFileName(basepath + filename);
     fileptr.open(QIODevice::WriteOnly);
-    fileptr.write(contents.toAscii());
+    fileptr.write(contents.toUtf8());
     fileptr.close();
-    KUrl url = KUrl(basepath + filename);
-    url.cleanPath();
-    kDebug() <<  "updating duchain for " << url.url() << basepath;
+    auto url = QUrl::fromLocalFile(QDir::cleanPath(basepath + filename));
+    qCDebug(KDEV_PYTHON_CODECOMPLETION) <<  "updating duchain for " << url.url() << basepath;
     const IndexedString urlstring(url);
     DUChain::self()->updateContextForUrl(urlstring, KDevelop::TopDUContext::ForceUpdate);
     ICore::self()->languageController()->backgroundParser()->parseDocuments();
@@ -87,21 +93,22 @@ void PyCompletionTest::initShell()
     AutoTestShell::init();
     TestCore* core = new TestCore();
     core->initialize(KDevelop::Core::NoUi);
-    fileEngine.mkdir(basepath, false);
-    
-    KUrl doc_url = KUrl(KStandardDirs::locate("data", "kdevpythonsupport/documentation_files/builtindocumentation.py"));
-    doc_url.cleanPath(KUrl::SimplifyDirSeparators);
-    
+    QDir d;
+    d.mkpath(basepath);
+
+    auto doc_url = QDir::cleanPath(QStandardPaths::locate(QStandardPaths::GenericDataLocation,
+                                                          "kdevpythonsupport/documentation_files/builtindocumentation.py"));
+
     DUChain::self()->updateContextForUrl(IndexedString(doc_url), KDevelop::TopDUContext::AllDeclarationsContextsAndUses);
     ICore::self()->languageController()->backgroundParser()->parseDocuments();
     DUChain::self()->waitForUpdate(IndexedString(doc_url), KDevelop::TopDUContext::AllDeclarationsContextsAndUses);
-    
+
     DUChain::self()->disablePersistentStorage();
     KDevelop::CodeRepresentation::setDiskChangesForbidden(true);
-    
+
     // now, create a nice little completion hierarchy
-    fileEngine.mkdir(basepath + "submoduledir", false);
-    fileEngine.mkdir(basepath + "submoduledir/anothersubdir", false);
+    d.mkpath(basepath + "submoduledir");
+    d.mkpath(basepath + "submoduledir/anothersubdir");
     makefile("toplevelmodule.py", "some_var = 3\ndef some_function(): pass\nclass some_class():\n def method(): pass");
     makefile("submoduledir/__init__.py", "var_in_sub_init = 5");
     makefile("submoduledir/subfile.py", "var_in_subfile = 5\nclass some_subfile_class():\n def method2(): pass");
@@ -112,8 +119,8 @@ void PyCompletionTest::initShell()
 
 void PyCompletionTest::testIdentifierMatching()
 {
-    QCOMPARE(camelCaseToUnderscore("FooBarBaz").toAscii().data(), "foo_bar_baz");
-    QCOMPARE(camelCaseToUnderscore("fooBarbaz").toAscii().data(),  "foo_barbaz");
+    QCOMPARE(camelCaseToUnderscore("FooBarBaz").toUtf8().data(), "foo_bar_baz");
+    QCOMPARE(camelCaseToUnderscore("fooBarbaz").toUtf8().data(),  "foo_barbaz");
 
     QCOMPARE(identifierMatchQuality("foobar", "foobar"),  3);
     QCOMPARE(identifierMatchQuality("foobar", "bar"),  2);
@@ -141,7 +148,7 @@ void PyCompletionTest::testExpressionParserMisc()
     QString calledFunction = p.popExpression(&s);
     QVERIFY(s == ExpressionParser::ExpressionFound);
     QCOMPARE(calledFunction, QString("foobar"));
-    
+
     ExpressionParser q("hello(world, foo.bar[3].foobar(3, \"some_string\", func(), funcfunc(3, 5), \t");
     q.skipUntilStatus(ExpressionParser::EventualCallFound, &ok, &expressionsSkipped);
     QVERIFY(ok);
@@ -157,7 +164,7 @@ void PyCompletionTest::testExpressionParser()
     QFETCH(QString, data);
     QFETCH(int, expectedStatus);
     QFETCH(QString, expectedExpression);
-    
+
     ExpressionParser p(data);
     ExpressionParser::Status status;
     QString result = p.popExpression(&status);
@@ -170,7 +177,7 @@ void PyCompletionTest::testExpressionParser_data()
     QTest::addColumn<QString>("data");
     QTest::addColumn<int>("expectedStatus");
     QTest::addColumn<QString>("expectedExpression");
-    
+
     QTest::newRow("attrExpression") << "foo.bar.baz" << (int) ExpressionParser::ExpressionFound << "foo.bar.baz";
     QTest::newRow("attrExpressionAccess") << "foo.bar.baz." << (int) ExpressionParser::MemberAccessFound << "";
     QTest::newRow("attrExpressionCall") << "foo.bar(3, 5, 7, hell0(3)).baz" << (int) ExpressionParser::ExpressionFound << "foo.bar(3, 5, 7, hell0(3)).baz";
@@ -183,41 +190,55 @@ void PyCompletionTest::testExpressionParser_data()
 
 const QList<CompletionTreeItem*> PyCompletionTest::invokeCompletionOn(const QString& initCode, const QString& invokeCode)
 {
+    CompletionParameters data = prepareCompletion(initCode, invokeCode);
+    return runCompletion(data);
+}
+
+const CompletionParameters PyCompletionTest::prepareCompletion(const QString& initCode, const QString& invokeCode)
+{
+    CompletionParameters completion_data;
+
     QString filename = nextFilename();
     QFile fileptr(filename);
     fileptr.open(QIODevice::WriteOnly);
-    fileptr.write(initCode.toAscii().replace("%INVOKE", ""));
+    fileptr.write(initCode.toUtf8().replace("%INVOKE", ""));
     fileptr.close();
-    
+
     DUChain::self()->updateContextForUrl(IndexedString(filename), KDevelop::TopDUContext::ForceUpdate);
     ICore::self()->languageController()->backgroundParser()->parseDocuments();
     ReferencedTopDUContext topContext = DUChain::self()->waitForUpdate(IndexedString(filename),
                                                                        KDevelop::TopDUContext::AllDeclarationsAndContexts);
-    
+
     Q_ASSERT(topContext);
-    
+
     Q_ASSERT(initCode.indexOf("%INVOKE") != -1);
     QString copy = initCode;
     QString allCode = copy.replace("%INVOKE", invokeCode);
-    
+
     QStringList lines = allCode.split('\n');
-    CursorInRevision cursorAt = CursorInRevision::invalid();
+    completion_data.cursorAt = CursorInRevision::invalid();
     for ( int i = 0; i < lines.length(); i++ ) {
-        if ( int j = lines.at(i).indexOf("%CURSOR") != -1 ) {
-            cursorAt = CursorInRevision(i, j);
+        int j = lines.at(i).indexOf("%CURSOR");
+        if ( j != -1 ) {
+            completion_data.cursorAt = CursorInRevision(i, j);
             break;
         }
     }
-    Q_ASSERT(cursorAt.isValid());
+    Q_ASSERT(completion_data.cursorAt.isValid());
     // codeCompletionContext only gets passed the text until the place where completion is invoked
-    QString snip = allCode.mid(0, allCode.indexOf("%CURSOR"));
-    QString remaining = allCode.mid(allCode.indexOf("%CURSOR"));
-    
+    completion_data.snip = allCode.mid(0, allCode.indexOf("%CURSOR"));
+    completion_data.remaining = allCode.mid(allCode.indexOf("%CURSOR") + 7);
+
     DUChainReadLocker lock;
-    DUContextPointer contextAtCursor = DUContextPointer(topContext->findContextAt(cursorAt, true));
-    Q_ASSERT(contextAtCursor);
-    
-    PythonCodeCompletionContext* context = new PythonCodeCompletionContext(contextAtCursor, snip, remaining, cursorAt, 0, 0);
+    completion_data.contextAtCursor = DUContextPointer(topContext->findContextAt(completion_data.cursorAt, true));
+    Q_ASSERT(completion_data.contextAtCursor);
+
+    return completion_data;
+}
+
+const QList<CompletionTreeItem*> PyCompletionTest::runCompletion(const CompletionParameters parameters)
+{
+    PythonCodeCompletionContext* context = new PythonCodeCompletionContext(parameters.contextAtCursor, parameters.snip, parameters.remaining, parameters.cursorAt, 0, 0);
     bool abort = false;
     QList<CompletionTreeItem*> items;
     foreach ( CompletionTreeItemPointer ptr, context->completionItems(abort, true) ) {
@@ -287,7 +308,7 @@ void PyCompletionTest::testImportCompletion_data()
     QTest::addColumn<QString>("invokeCode");
     QTest::addColumn<QString>("completionCode");
     QTest::addColumn<QString>("expectedItem");
-    
+
     QTest::newRow("same_directory") << "%INVOKE" << "import %CURSOR" << "toplevelmodule";
 //     QTest::newRow("same_directory_beginText") << "%INVOKE" << "import toplevelmo%CURSOR" << "toplevelmodule";
     QTest::newRow("nocompletion") << "%INVOKE" << "from toplevelmodule %CURSOR" << "EMPTY";
@@ -317,7 +338,7 @@ void PyCompletionTest::testCompletionAfterQuotes_data()
 {
     QTest::addColumn<QString>("invokeCode");
     QTest::addColumn<QString>("completionCode");
-    
+
     QTest::newRow("nothing") << "\n%INVOKE" << "%CURSOR";
     QTest::newRow("sq_in_string") << "\"foo'bar\"\n%INVOKE" << "%CURSOR";
     QTest::newRow("sq_in_sl_comment") << "#foo'bar\n%INVOKE" << "%CURSOR";
@@ -339,7 +360,7 @@ void PyCompletionTest::testIntegralTypesImmediate()
     QFETCH(QString, invokeCode);
     QFETCH(QString, completionCode);
     QFETCH(QString, expectedDeclaration);
-    
+
     QVERIFY(declarationInCompletionList(invokeCode, completionCode, expectedDeclaration));
 }
 
@@ -348,7 +369,7 @@ void PyCompletionTest::testIntegralTypesImmediate_data()
     QTest::addColumn<QString>("invokeCode");
     QTest::addColumn<QString>("completionCode");
     QTest::addColumn<QString>("expectedDeclaration");
-    
+
     QTest::newRow("list_syntax") << "[]%INVOKE" << ".%CURSOR" << "append";
     QTest::newRow("dict_syntax") << "{}%INVOKE" << ".%CURSOR" << "items";
     QTest::newRow("string_syntax") << "\"\"%INVOKE" << ".%CURSOR" << "capitalize";
@@ -362,7 +383,7 @@ void PyCompletionTest::testIntegralExpressionsDifferentContexts()
     QFETCH(QString, invokeCode);
     QFETCH(QString, completionCode);
     QFETCH(QString, expectedDeclaration);
-    
+
     QVERIFY(declarationInCompletionList(invokeCode, completionCode, expectedDeclaration));
 }
 
@@ -371,14 +392,14 @@ void PyCompletionTest::testIntegralExpressionsDifferentContexts_data()
     QTest::addColumn<QString>("invokeCode");
     QTest::addColumn<QString>("completionCode");
     QTest::addColumn<QString>("expectedDeclaration");
-    
+
     QTest::newRow("function_call") << "foo([]%INVOKE)" << ".%CURSOR" << "append";
     QTest::newRow("function_call_multi") << "foo(bar(baz(bang([]%INVOKE))))" << ".%CURSOR" << "append";
     QTest::newRow("empty_list") << "[[]%INVOKE]" << ".%CURSOR" << "append";
     QTest::newRow("list") << "[1, 2, 3, 4, 5, []%INVOKE]" << ".%CURSOR" << "append";
     QTest::newRow("list_with_fancy_string") << "[\"FooFObar\\\", 3)\", []%INVOKE]" << ".%CURSOR" << "append";
     QTest::newRow("empty_dict") << "{[]%INVOKE}" << ".%CURSOR" << "append";
-    QTest::newRow("print_stmt") << "print []%INVOKE" << ".%CURSOR" << "append";
+    QTest::newRow("print_stmt") << "%INVOKE" << "print([].%CURSOR" << "append";
 }
 
 void PyCompletionTest::testIgnoreCommentSignsInStringLiterals()
@@ -392,7 +413,7 @@ void PyCompletionTest::testNoCompletionInCommentsOrStrings()
 {
     QFETCH(QString, invokeCode);
     QFETCH(QString, completionCode);
-    
+
     QVERIFY(! declarationInCompletionList(invokeCode, completionCode, "append"));
 }
 
@@ -400,7 +421,7 @@ void PyCompletionTest::testNoCompletionInCommentsOrStrings_data()
 {
     QTest::addColumn<QString>("invokeCode");
     QTest::addColumn<QString>("completionCode");
-    
+
     QTest::newRow("single_comment") << "# []%INVOKE" << ".%CURSOR";
     QTest::newRow("single_comment_local") << "local=3\n# %INVOKE" << "%CURSOR";
     QTest::newRow("stringDQ") << "\"[]%INVOKE\"" << ".%CURSOR";
@@ -422,7 +443,7 @@ void PyCompletionTest::testImplementMethodCompletion_data()
 {
     QTest::addColumn<QString>("invokeCode");
     QTest::addColumn<QString>("completionCode");
-    
+
     QTest::newRow("simple_begin") << "class myclass():\n %INVOKE\n pass" << "def %CURSOR";
     QTest::newRow("another_method_before") << "class myclass():\n def some_method(param):pass\n %INVOKE" << "def %CURSOR";
     QTest::newRow("another_method_before_multiline") << "class myclass():\n def some_method(param):\n  pass\n  pass \n  pass"
@@ -450,8 +471,9 @@ void PyCompletionTest::testAutoBrackets()
     KService::Ptr documentService = KService::serviceByDesktopPath("katepart.desktop");
     QVERIFY(documentService);
     KTextEditor::Document* document = documentService->createInstance<KTextEditor::Document>(this);
+    auto view = document->createView(nullptr);
     QVERIFY(document);
-    item->execute(document, KTextEditor::Range(0, 0, 0, 0));
+    item->execute(view, KTextEditor::Range(0, 0, 0, 0));
     QCOMPARE(document->text(), QLatin1String("myprop"));
 }
 
@@ -460,7 +482,7 @@ void PyCompletionTest::testExceptionCompletion()
     QList< CompletionTreeItem* > items = invokeCompletionOn("localvar = 3\nraise %INVOKE", "%CURSOR");
     QVERIFY(containsItemForDeclarationNamed(items, "Exception"));
     QVERIFY(! containsItemForDeclarationNamed(items, "localvar"));
-    
+
     items = invokeCompletionOn("localvar = 3\n%INVOKE", "try: pass\nexcept %CURSOR");
     QVERIFY(containsItemForDeclarationNamed(items, "Exception"));
     QVERIFY(! containsItemForDeclarationNamed(items, "localvar"));
@@ -469,6 +491,8 @@ void PyCompletionTest::testExceptionCompletion()
 void PyCompletionTest::testGeneratorCompletion()
 {
     QVERIFY(itemInCompletionList("%INVOKE", "foobar = [item for %CURSOR", "item in"));
+    QVERIFY(itemInCompletionList("%INVOKE", "foobar = [key, value for %CURSOR", "key, value in"));
+    QVERIFY(itemInCompletionList("%INVOKE", "foobar = [str(key + value) for %CURSOR", "key, value in"));
     QVERIFY(itemInCompletionList("%INVOKE\ndec_l8r=3", "foobar = [dec_l8r for %CURSOR", "dec_l8r in"));
 }
 
@@ -502,6 +526,171 @@ void PyCompletionTest::testAddImportCompletion_data()
     QTest::newRow("has_no_when_not_necessary") << "toplevelmodule = 3;\ntoplevelmodule%INVOKE" << ".%CURSOR" << 0;
 }
 
+void PyCompletionTest::testFunctionDeclarationCompletion()
+{
+    QFETCH(QString, completionCode);
+    QFETCH(QString, invokeCode);
+    QFETCH(KTextEditor::Range, executeRange);
+    QFETCH(QString, expectedReplacement);
+
+    QString documentCode = completionCode;
+    documentCode.replace("%INVOKE", invokeCode).replace("%CURSOR", "");
+
+    QString expectedCode = completionCode;
+    expectedCode.replace("%INVOKE", expectedReplacement);
+
+    const QList<CompletionTreeItem *> completionItems = invokeCompletionOn(completionCode, invokeCode);
+
+    QVERIFY( ! completionItems.isEmpty() );
+
+    KService::Ptr documentService = KService::serviceByDesktopPath("katepart.desktop");
+    QVERIFY(documentService);
+    KTextEditor::Document* document = documentService->createInstance<KTextEditor::Document>(this);
+    QVERIFY(document);
+    document->setText(documentCode);
+
+    auto view = document->createView(nullptr);
+
+    completionItems.first()->execute(view, executeRange);
+    QCOMPARE(document->text(), expectedCode);
 }
 
+void PyCompletionTest::testFunctionDeclarationCompletion_data()
+{
+    QTest::addColumn<QString>("completionCode");
+    QTest::addColumn<QString>("invokeCode");
+    QTest::addColumn<KTextEditor::Range>("executeRange");
+    QTest::addColumn<QString>("expectedReplacement");
 
+    QTest::newRow("func_decl_no_parens") << "def foo():\n  pass\n%INVOKE" << "foo%CURSOR" << KTextEditor::Range(2, 0, 2, 3)
+                                         << "foo()";
+
+    QTest::newRow("func_decl_existing_parens") << "def foo():\n  return 0\nbar = %INVOKE" << "foo%CURSOR()" << KTextEditor::Range(2, 6, 2, 9)
+                                           << "foo()";
+
+    QTest::newRow("decorator_no_parens") << "def mydecorator():\n  pass\nclass Foo:\n  %INVOKE\n  def bar():\n    pass" << "@mydecorator%CURSOR" << KTextEditor::Range(3, 3, 3, 15)
+                                         << "@mydecorator";
+
+    QTest::newRow("class_name_no_constructor_parens") << "class Foo:\n  pass\nbar = %INVOKE" << "Foo%CURSOR" << KTextEditor::Range(2, 6, 2, 9)
+                                                      << "Foo()";
+
+    QTest::newRow("class_name_explicit_constructor_parens") << "class Foo:\n  def __init__(self):\n    pass\nbar = %INVOKE" << "Fo%CURSOR"
+                                                        << KTextEditor::Range(3, 6, 3, 9)
+                                                        << "Foo()";
+}
+
+void PyCompletionTest::testStringFormattingCompletion()
+{
+    QFETCH(QString, completionCode);
+    QFETCH(QString, invokeCode);
+    QFETCH(QString, expectedItem);
+    QFETCH(bool, expectedPresent);
+
+    QCOMPARE(itemInCompletionList(completionCode, invokeCode, expectedItem), expectedPresent);
+}
+
+void PyCompletionTest::testStringFormattingCompletion_data()
+{
+    QTest::addColumn<QString>("completionCode");
+    QTest::addColumn<QString>("invokeCode");
+    QTest::addColumn<QString>("expectedItem");
+    QTest::addColumn<bool>("expectedPresent");
+
+    QTest::newRow("sq_string") << "'foo %INVOKE'" << "%CURSOR" << "{0}" << true;
+    QTest::newRow("dq_string") << "\"foo %INVOKE bar\"" << "%CURSOR" << "{0}" << true;
+    QTest::newRow("sq_ml_string") << "'''foo\n\n%INVOKE\nbar'''" << "%CURSOR" << "{0}" << true;
+    QTest::newRow("dq_ml_string") << "\"\"\"foo\nbar %INVOKE baz\"\"\"" << "%CURSOR" << "{0}" << true;
+    QTest::newRow("auto_id") << "\"foo {0} bar {1} baz %INVOKE\"" << "%CURSOR" << "{2}" << true;
+
+    QTest::newRow("format_suggestions_conversion") << "\"foo {0} bar %INVOKE\"" << "{1}%CURSOR" << "{1!r}" << true;
+    QTest::newRow("format_suggestions_spec") << "\"foo {0} bar {1}%INVOKE baz\"" << "{2}%CURSOR" << "{2:%}" << true;
+
+    QTest::newRow("incompatible_suggestions_conversion") << "\"foo %INVOKE bar\"" << "{0:%}%CURSOR" << "{0!s:%}" << false;
+    QTest::newRow("incompatible_suggestions_spec") << "\"foo %INVOKE bar\"" << "{0!s}%CURSOR" << "{0!s:%}" << false;
+
+    QTest::newRow("alignment_for_strings") << "\"foo %INVOKE bar\"" << "{0!s}%CURSOR" << "{0!s:^${width}}" << true;
+    QTest::newRow("conversion_for_aliged_strings") << "\"foo %INVOKE bar\"" << "{0!s}%CURSOR" << "{0!s:^${width}}" << true;
+
+}
+
+void PyCompletionTest::testStringFormatter()
+{
+    QFETCH(QString, string);
+    QFETCH(int, expectedId);
+    QFETCH(QList<RangeInString>, expectedVariablePositions);
+
+    StringFormatter f(string);
+
+    int id = f.nextIdentifierId();
+    QCOMPARE(id, expectedId);
+
+    for (int i = 0; i < string.size(); i++) {
+        bool expectedInsideVariable = false;
+        foreach (RangeInString range, expectedVariablePositions) {
+            if (i >= range.beginIndex && i <= range.endIndex) {
+                expectedInsideVariable = true;
+                break;
+            }
+        }
+        QCOMPARE(f.isInsideReplacementVariable(i), expectedInsideVariable);
+    }
+}
+
+void PyCompletionTest::testStringFormatter_data()
+{
+    QTest::addColumn<QString>("string");
+    QTest::addColumn<int>("expectedId");
+    QTest::addColumn<QList<RangeInString> >("expectedVariablePositions");
+
+    QTest::newRow("sl_string") << "\"foo {0} bar {1}\"" << 2
+                               << (QList<RangeInString>() << RangeInString(5, 8) << RangeInString(13, 16));
+
+    QTest::newRow("ml_string") << "\"\"\"foo {0} \n\nbar {1} \n{foo} {2}\nbaz\"\"\"" << 3
+                               << (QList<RangeInString>() << RangeInString(7, 10) << RangeInString(17, 20)
+                                   << RangeInString(22, 27) << RangeInString(28, 31));
+
+    QTest::newRow("containing_quotes") << "'''foo {0}\nbar\n{1}{0} \\' \\\" \\\" {2} {foo} \n\\'\n {3}baz'''" << 4
+                                       << (QList<RangeInString>() << RangeInString(7, 10) << RangeInString(15, 18)
+                                           << RangeInString(18, 21) << RangeInString(31, 34) << RangeInString(35, 40)
+                                           << RangeInString(46, 49));
+}
+
+QString repeat_distinct(const QString& code, int count) {
+    QString result;
+    QString line;
+    for ( int i = 0; i < count; i++ ) {
+        line = code;
+        result.append(line.replace(QString("%X"), QString::number(i)));
+    }
+    return result;
+}
+
+void PyCompletionTest::completionBenchTest()
+{
+    QFETCH(QString, completionCode);
+    QFETCH(QString, invokeCode);
+
+    CompletionParameters data = prepareCompletion(completionCode, invokeCode);
+    QBENCHMARK {
+        runCompletion(data);
+    }
+}
+
+void PyCompletionTest::completionBenchTest_data()
+{
+    QTest::addColumn<QString>("completionCode");
+    QTest::addColumn<QString>("invokeCode");
+
+    QTest::newRow("variable_completion") << "a0 = 2\n%INVOKE" << "b = a%CURSOR";
+    QTest::newRow("no_items") << "%INVOKE" << "def func(%CURSOR";
+    QTest::newRow("function") << "%INVOKE" << "foo(%CURSOR";
+    QTest::newRow("deep_function") << "foo(bar(baz(bang([]%INVOKE))))" << ".%CURSOR";
+    QTest::newRow("class_completion") << "class my(): pass\nd = my()\n%INVOKE" << "d.%CURSOR";
+
+    QString many_globals = repeat_distinct("a%X=%X\n", 1000);
+
+    QTest::newRow("function_many_globals") << many_globals + "%INVOKE" << "foo(%CURSOR";
+    QTest::newRow("variable_completion_many_globals") << many_globals + "%INVOKE" << "b = a%CURSOR";
+}
+
+}
