@@ -2,6 +2,7 @@
  *   This file is part of KDevelop                                         *
  *   Copyright 2007 Andreas Pakulat <apaku@gmx.de>                         *
  * Copyright 2010 Sven Brauch <svenbrauch@googlemail.com>                  *
+ *   Copyright 2012 Patrick Spendrin <ps_ml@gmx.de>                        *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -21,31 +22,63 @@
 
 #include "astdefaultvisitor.h"
 #include "ast.h"
-#include <kdebug.h>
 
 namespace Python
 {
 
+void free_ast_recursive(CodeAst *node)
+{
+    AstFreeVisitor v;
+    v.visitCode(node);
+}
+
 AstDefaultVisitor::AstDefaultVisitor() { }
 AstDefaultVisitor::~AstDefaultVisitor() { }
 
 // The Ast "ends" here, those dont have child nodes
 // note that Identifier is not a node in this Ast
-void AstDefaultVisitor::visitName(NameAst* node) { Q_UNUSED(node); }
+void AstDefaultVisitor::visitNameConstant(NameConstantAst* node) { Q_UNUSED(node); }
 void AstDefaultVisitor::visitPass(PassAst* node) { Q_UNUSED(node); }
-void AstDefaultVisitor::visitAlias(AliasAst* node) { Q_UNUSED(node); }
+void AstDefaultVisitor::visitNonlocal(NonlocalAst* node) { Q_UNUSED(node); }
 void AstDefaultVisitor::visitBreak(BreakAst* node) { Q_UNUSED(node); }
 void AstDefaultVisitor::visitContinue(ContinueAst* node) { Q_UNUSED(node); }
 void AstDefaultVisitor::visitEllipsis(EllipsisAst* node) { Q_UNUSED(node); }
-void AstDefaultVisitor::visitGlobal(GlobalAst* node) { Q_UNUSED(node); }
 void AstDefaultVisitor::visitNumber(NumberAst* node) { Q_UNUSED(node); }
 void AstDefaultVisitor::visitString(StringAst* node) { Q_UNUSED(node); }
+void AstDefaultVisitor::visitBytes(BytesAst* node) { Q_UNUSED(node); }
+void AstDefaultVisitor::visitIdentifier(Identifier* node) { Q_UNUSED(node); }
+
+void AstDefaultVisitor::visitStarred(StarredAst* node) {
+    visitNode(node->value);
+}
+
+void AstDefaultVisitor::visitArg(ArgAst* node) {
+    visitNode(node->annotation);
+    visitNode(node->argumentName);
+    visitIdentifier(node->argumentName);
+}
+
+void AstDefaultVisitor::visitAlias(AliasAst* node) {
+    visitIdentifier(node->name);
+    visitIdentifier(node->asName);
+}
+
+void AstDefaultVisitor::visitName(NameAst* node) {
+    visitIdentifier(node->identifier);
+}
+
+void AstDefaultVisitor::visitGlobal(GlobalAst* node) {
+    foreach (Identifier* identifier, node->names) {
+        visitIdentifier(identifier);
+    }
+}
 
 void AstDefaultVisitor::visitCode(CodeAst* node)
 {
     foreach (Ast* statement, node->body) {
         visitNode(statement);
     }
+    visitIdentifier(node->name);
 }
 
 void AstDefaultVisitor::visitExpression(ExpressionAst* node)
@@ -53,6 +86,11 @@ void AstDefaultVisitor::visitExpression(ExpressionAst* node)
     visitNode(node->value);
 }
 
+void AstDefaultVisitor::visitYieldFrom(YieldFromAst* node)
+{
+    visitNode(node->value);
+}
+
 void AstDefaultVisitor::visitAssertion(AssertionAst* node)
 {
     visitNode(node->condition);
@@ -66,13 +104,6 @@ void AstDefaultVisitor::visitDelete(DeleteAst* node)
     }
 }
 
-void AstDefaultVisitor::visitExec(ExecAst* node)
-{
-    visitNode(node->body);
-    visitNode(node->globals);
-    visitNode(node->locals);
-}
-
 void AstDefaultVisitor::visitExtendedSlice(ExtendedSliceAst* node)
 {
     foreach (SliceAst* slice, node->dims) {
@@ -130,6 +161,7 @@ void AstDefaultVisitor::visitImportFrom(ImportFromAst* node)
     foreach (AliasAst* alias, node->names) {
         visitNode(alias);
     }
+    visitIdentifier(node->module);
 }
 
 void AstDefaultVisitor::visitIndex(IndexAst* node)
@@ -148,13 +180,12 @@ void AstDefaultVisitor::visitRaise(RaiseAst* node)
     visitNode(node->type);
 }
 
-void AstDefaultVisitor::visitRepr(ReprAst* node)
+void AstDefaultVisitor::visitReturn(ReturnAst* node)
 {
     visitNode(node->value);
 }
 
-void AstDefaultVisitor::visitReturn(ReturnAst* node)
-{
+void AstDefaultVisitor::visitAwait(AwaitAst* node) {
     visitNode(node->value);
 }
 
@@ -186,7 +217,7 @@ void AstDefaultVisitor::visitSubscript(SubscriptAst* node)
     visitNode(node->slice);
 }
 
-void AstDefaultVisitor::visitTryExcept(TryExceptAst* node)
+void AstDefaultVisitor::visitTry(TryAst* node)
 {
     foreach (Ast* statement, node->body) {
         visitNode(statement);
@@ -197,14 +228,7 @@ void AstDefaultVisitor::visitTryExcept(TryExceptAst* node)
     foreach (Ast* statement, node->orelse) {
         visitNode(statement);
     }
-}
-
-void AstDefaultVisitor::visitTryFinally(TryFinallyAst* node)
-{
-    foreach (Ast* statement, node->body) {
-        visitNode(statement);
-    }
-    foreach (Ast* statement, node->finalbody) {
+    foreach (Ast* statement, node->finally) {
         visitNode(statement);
     }
 }
@@ -234,13 +258,20 @@ void AstDefaultVisitor::visitWhile(WhileAst* node)
 
 void AstDefaultVisitor::visitWith(WithAst* node)
 {
-    visitNode(node->contextExpression);
-    visitNode(node->optionalVars);
+    foreach (Ast* item, node->items) {
+        visitNode(item);
+    }
     foreach (Ast* statement, node->body) {
         visitNode(statement);
     }
 }
 
+void AstDefaultVisitor::visitWithItem(WithItemAst* node)
+{
+    visitNode(node->contextExpression);
+    visitNode(node->optionalVars);
+}
+
 void AstDefaultVisitor::visitYield(YieldAst* node)
 {
     visitNode(node->value);
@@ -319,6 +350,7 @@ void AstDefaultVisitor::visitClassDefinition(ClassDefinitionAst* node)
     foreach (ExpressionAst* expression, node->decorators) {
         visitNode(expression);
     }
+    visitIdentifier(node->name);
 }
 
 void AstDefaultVisitor::visitCompare(CompareAst* node)
@@ -346,19 +378,9 @@ void AstDefaultVisitor::visitAssignment(AssignmentAst* node)
     visitNode(node->value);
 }
 
-void AstDefaultVisitor::visitPrint(PrintAst* node)
-{
-    visitNode(node->destination);
-    foreach (ExpressionAst* expression, node->values) {
-        visitNode(expression);
-    }
-}
-
 void AstDefaultVisitor::visitCall(CallAst* node)
 {
     visitNode(node->function);
-    visitNode(node->keywordArguments);
-    visitNode(node->starArguments);
     foreach (ExpressionAst* argument, node->arguments) {
         visitNode(argument);
     }
@@ -373,24 +395,28 @@ void AstDefaultVisitor::visitFunctionDefinition(FunctionDefinitionAst* node)
         visitNode(decorator);
     }
     visitNode(node->arguments);
+    visitNode(node->returns);
     foreach (Ast* stmt, node->body) {
         visitNode(stmt);
     }
+    visitIdentifier(node->name);
 }
 
 void AstDefaultVisitor::visitAttribute(AttributeAst* node)
 {
     visitNode(node->value);
+    visitIdentifier(node->attribute);
 }
 
 void AstDefaultVisitor::visitKeyword(KeywordAst* node)
 {
     visitNode(node->value);
+    visitIdentifier(node->argumentName);
 }
 
 void AstDefaultVisitor::visitArguments(ArgumentsAst* node)
 {
-    foreach (ExpressionAst* expression, node->arguments) {
+    foreach (ArgAst* expression, node->arguments) {
         visitNode(expression);
     }
     foreach (ExpressionAst* expression, node->defaultValues ) {
